package com.xs.core.config.mq;

import org.springframework.amqp.core.*;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.HashMap;
import java.util.Map;

@Configuration
public class RabbitMQConfig {
    public static final String GOLD_COIN_CALCULATION_QUEUE = "gold.coin.calculation.queue";
    public static final String GOLD_COIN_CALCULATION_EXCHANGE = "gold.coin.calculation.exchange";
    public static final String GOLD_COIN_COMPLETE_QUEUE = "gold.coin.complete.queue";
    public static final String GOLD_COIN_COMPLETE_EXCHANGE = "gold.coin.complete.exchange";

    @Bean
    public Queue goldCoinCompleteQueue() {
        return new Queue(GOLD_COIN_COMPLETE_QUEUE, true, false, false);
    }

    @Bean
    public DirectExchange goldCoinCompleteExchange() {
        return new DirectExchange(GOLD_COIN_COMPLETE_EXCHANGE, true, false);
    }

    @Bean
    public Binding bindingComplete() {
        return BindingBuilder.bind(goldCoinCompleteQueue())
                .to(goldCoinCompleteExchange())
                .with(GOLD_COIN_COMPLETE_QUEUE);
    }

    @Bean
    public CustomExchange delayExchange() {
        Map<String, Object> args = new HashMap<>();
        args.put("x-delayed-type", "direct");
        return new CustomExchange(GOLD_COIN_CALCULATION_EXCHANGE, "x-delayed-message", true, false, args);
    }

    @Bean
    public Queue goldCoinCalculationQueue() {
        return new Queue(GOLD_COIN_CALCULATION_QUEUE, true, false, false);
    }

    @Bean
    public Binding bindingCalculation() {
        return BindingBuilder.bind(goldCoinCalculationQueue())
                .to(delayExchange())
                .with(GOLD_COIN_CALCULATION_QUEUE)
                .noargs();
    }

}
