package com.xs.core.controller.coin;

import com.xs.core.model.ResponseResult;
import com.xs.core.model.coin.entity.CoinSpeedUpgradesRules;
import com.xs.core.model.coin.entity.GoldCoinProdState;
import com.xs.core.model.coin.req.CoinSpeedUpgradesRulesReq;
import com.xs.core.model.coin.req.TemporaryRateReq;
import com.xs.core.model.coin.resp.BoostResidueTimesResp;
import com.xs.core.model.coin.resp.CoinSpeedUpgradesRulesResp;
import com.xs.core.model.coin.resp.GoldCoinProdStateResp;
import com.xs.core.service.coin.GoldCoinService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/gold/coin/product")
@Tag(name = "金币产出相关api")
public class GoldCoinProductController {
    @Autowired
    private GoldCoinService goldCoinService;

    @Operation(description = "开始产出金币", summary = "开始产出金币")
    @PostMapping("/start")
    public ResponseResult<?> startGoldCoinProduct() {
        goldCoinService.startProductGoldCoin();
        return ResponseResult.success();
    }

    @PostMapping("/getGoldCoinProductState")
    @Operation(description = "获取金币产出状态", summary = "获取金币产出状态")
    public ResponseResult<GoldCoinProdStateResp> getGoldCoinProductState() {
        GoldCoinProdStateResp goldCoinProdState = goldCoinService.getGoldCoinProdState();
        return ResponseResult.success(goldCoinProdState);
    }


    @PostMapping("/updateGoldCoinProductRate")
    @Operation(description = "提升金币产出永久速率", summary = "提升金币产出永久速率")
    public ResponseResult<?> updateGoldCoinProductPermanentRate(@RequestBody CoinSpeedUpgradesRulesReq req) {
        goldCoinService.permanentBoostProductGoldCoinRate(req);
        return ResponseResult.success();
    }

    @PostMapping("/getRateUpgradesRules")
    @Operation(description = "获取金币产出速率升级规则", summary = "获取金币产出速率升级规则")
    public ResponseResult<CoinSpeedUpgradesRulesResp> getRateUpgradesRules() {
        CoinSpeedUpgradesRulesResp rules = goldCoinService.getRateUpgradesRulesByUser();
        return ResponseResult.success(rules);
    }

    @Operation(description = "更新临时速率", summary = "更新临时速率")
    @PostMapping("/updateGoldCoinProductTemporaryRate")
    public ResponseResult<?> updateGoldCoinProductTemporaryRate(@RequestBody TemporaryRateReq req) {
        goldCoinService.temporaryBoostProductGoldCoinRate(req);
        return ResponseResult.success();
    }

    @Operation(description = "收取产出的币", summary = "收取产出的币")
    @PostMapping("/collectGoldCoin")
    public ResponseResult<?> collectGoldCoin() {
        goldCoinService.collectGoldCoin();
        return ResponseResult.success();
    }

    @Operation(description = "加成剩余时间", summary = "加成剩余时间")
    @PostMapping("/getBoostResidueTimes")
    public ResponseResult<BoostResidueTimesResp> getBoostResidueTimes() {
        BoostResidueTimesResp boostResidueTimesResp = goldCoinService.getBoostResidueTimes();
        return ResponseResult.success(boostResidueTimesResp);
    }
}
