package com.xs.core.filter;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xs.core.common.constant.ConstantConfig;
import com.xs.core.common.content.UserContext;
import com.xs.core.common.content.UserContextHolder;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.servlet.HandlerInterceptor;

import java.util.Optional;

/**
 * 用户上下文拦截器
 */
@Slf4j
public class UserContextInterceptor implements HandlerInterceptor {
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        String extra = (String) StpUtil.getExtra(ConstantConfig.USER_CONTENT_EXTRA_KEY);
        if (StrUtil.isNotBlank(extra)) {
            UserContext userContext = JSON.parseObject(extra, UserContext.class);
            //设置语言
            String languageCode = Optional.ofNullable(request.getHeader("languageCode")).orElse("zh_CN");
            userContext.setLanguageCode(languageCode);
            UserContextHolder.setContext(userContext);
        }
        return HandlerInterceptor.super.preHandle(request, response, handler);
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) {
        //清除上下文防止内存溢出
        UserContextHolder.clearContext();
    }
}
