package com.xs.core.mq.consumer;

import com.alibaba.fastjson2.JSON;
import com.rabbitmq.client.Channel;
import com.xs.core.common.constant.GoldCoinConstant;
import com.xs.core.config.mq.RabbitMQConfig;
import com.xs.core.model.coin.entity.GoldCoinProdRecord;
import com.xs.core.model.coin.entity.GoldCoinProdState;
import com.xs.core.model.coin.msg.CoinProducerMessage;
import com.xs.core.service.coin.GoldCoinService;
import com.xs.core.service.coin.IGoldCoinProdRecordService;
import com.xs.core.service.coin.IGoldCoinProdStateService;
import com.xs.core.service.redis.RedisService;
import jdk.jfr.Label;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.time.LocalDateTime;

/**
 * 金币产出完成消息消费者
 */
@Component
@Slf4j
public class GoldCoinCompleteMessageConsumer {
    @Autowired
    private RedisService redisService;

    @Autowired
    @Lazy
    private GoldCoinService goldCoinService;

    @RabbitListener(queues = RabbitMQConfig.GOLD_COIN_COMPLETE_QUEUE, concurrency = "10")
    public void handleComplete(Message m, Channel channel) throws IOException {
        log.info("Received handleComplete message: {}", new String(m.getBody()));
        // 处理完成逻辑
        // 可以在这里更新数据库中的金币产出状态为完成
        String msg = new String(m.getBody());
        CoinProducerMessage coinProducerMessage = JSON.parseObject(msg, CoinProducerMessage.class);
        if (coinProducerMessage != null) {
            // 从Redis中获取金币产出状态
            GoldCoinProdState coinProdState = redisService.get(GoldCoinConstant.GOLD_COIN_STATE_KEY + coinProducerMessage.getUserId(), GoldCoinProdState.class);
            if (coinProdState != null) {
                // 执行金币结算逻辑
                goldCoinService.completedGoldCoinSettlement(coinProdState);
            }
        }
        channel.basicAck(m.getMessageProperties().getDeliveryTag(), false);
    }
}
