package com.xs.core.service.Impl.coin;

import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xs.core.common.constant.GoldCoinConstant;
import com.xs.core.model.coin.entity.GoldCoinProdRecord;
import com.xs.core.model.coin.entity.GoldCoinProdState;
import com.xs.core.mapper.coin.GoldCoinProdStateMapper;
import com.xs.core.model.coin.resp.GoldCoinProdStateResp;
import com.xs.core.service.coin.IGoldCoinProdRecordService;
import com.xs.core.service.coin.IGoldCoinProdStateService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xs.core.service.redis.RedisService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 金币生产状态 服务实现类
 * </p>
 *
 * @author xudm
 * @since 2024-12-18
 */
@Service
public class GoldCoinProdStateServiceImpl extends ServiceImpl<GoldCoinProdStateMapper, GoldCoinProdState> implements IGoldCoinProdStateService {

    @Autowired
    private RedisService redisService;

    @Autowired
    private IGoldCoinProdRecordService recordService;

    @Override
    public GoldCoinProdState getGoldCoinProdStateByUser(Long userId) {
        //先尝试获取redis中的值
        GoldCoinProdState coinProdState = redisService.get(GoldCoinConstant.GOLD_COIN_STATE_KEY + userId, GoldCoinProdState.class);
        if (ObjUtil.isNotNull(coinProdState)) {
            return coinProdState;
        } else {
            //redis中没有值，尝试获取数据库中的值
            LambdaQueryWrapper<GoldCoinProdState> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(GoldCoinProdState::getUserId, userId);
            return getOne(queryWrapper);
        }
    }

    @Override
    public GoldCoinProdStateResp getGoldCoinProdRespStateByUser(Long userId) {
        GoldCoinProdState state = getGoldCoinProdStateByUser(userId);
        if (ObjUtil.isNotNull(state)) {
            GoldCoinProdStateResp resp = new GoldCoinProdStateResp();
            BeanUtils.copyProperties(state, resp);
            if (Boolean.TRUE.equals(state.getRunning())) {
                resp.setStatus(1);
            } else {
                //判断是否存在待领取的金币
                GoldCoinProdRecord record = recordService.getNoReciviedGoldCoinProdRecordByUserId(userId);
                if (ObjUtil.isNotNull(record)) {
                    resp.setStatus(2);
                } else {
                    resp.setStatus(0);
                }
            }
            return resp;
        }
        return null;
    }
}
