package com.xs.core.service.Impl.user;

import cn.dev33.satoken.stp.SaLoginConfig;
import cn.dev33.satoken.stp.SaTokenInfo;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xs.core.common.constant.ConstantConfig;
import com.xs.core.common.content.UserContext;
import com.xs.core.common.enums.DisEnableStatusEnum;
import com.xs.core.common.validation.CheckUtils;
import com.xs.core.model.user.entity.TgUser;
import com.xs.core.model.user.req.AppLoginReq;
import com.xs.core.service.user.AppLoginService;
import com.xs.core.service.user.ITgUserService;
import com.xs.core.utils.InviteCodeGenerator;
import com.xs.core.utils.SpringWebUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.concurrent.CompletableFuture;

@Service
@AllArgsConstructor
@Slf4j
public class AppLoginServiceImpl implements AppLoginService {
    private final ITgUserService userService;

    @Override
    public SaTokenInfo tgLogin(AppLoginReq loginReq) {
        LambdaQueryWrapper<TgUser> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TgUser::getTgId, loginReq.getId());
        TgUser user = userService.getOne(queryWrapper);
        if (ObjUtil.isNull(user)) {
            user = registerToSys(loginReq);
        }
        //校验是否被禁用
        checkUserStatus(user);
        return this.login(user);
    }

    @Override
    public TgUser registerToSys(AppLoginReq loginReq) {
        TgUser user = TgUser.builder()
                .tgAccount(loginReq.getUsername())
                .tgId(loginReq.getId())
                .avatar(StrUtil.isNotBlank(loginReq.getPhotoUrl()) ? URLDecoder.decode(loginReq.getPhotoUrl(), StandardCharsets.UTF_8) : "")
                .firstName(loginReq.getFirstName())
                .lastName(loginReq.getLastName())
                //注册时指定用户的唯一邀请码
                .inviteCode(InviteCodeGenerator.generateInviteCode())
                .languageCode(loginReq.getLanguageCode())
                .createdTime(LocalDateTime.now())
                .updatedTime(LocalDateTime.now())
                .build();
        if (StrUtil.isNotBlank(loginReq.getAge_limit())) {
            user.setAgeLimit(Integer.parseInt(loginReq.getAge_limit()));
        }
        if (StrUtil.isNotBlank(loginReq.getShare_code())) {
            //如邀请码不为空 则写入邀请人信息
            LambdaQueryWrapper<TgUser> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(TgUser::getInviteCode, loginReq.getShare_code());
            TgUser inviteUser = userService.getOne(queryWrapper);
            if (ObjUtil.isNotNull(inviteUser)) {
                user.setReferrerId(String.valueOf(inviteUser.getId()));
            }
            user.setPassengerFlowWay(2);
        }
        userService.save(user);
        return user;
    }

    /**
     * 检查用户状态
     *
     * @param user 用户信息
     */
    private void checkUserStatus(TgUser user) {
        CheckUtils.throwIfEqual(DisEnableStatusEnum.DISABLE.getValue(), user.getDisableFlag(), "此账号已被禁用，如有疑问，请联系管理员");
    }

    private SaTokenInfo login(TgUser user) {
        Long userId = user.getId();
        user.setLoginTime(LocalDateTime.now());
        UserContext userContext = new UserContext();
        BeanUtils.copyProperties(user, userContext);
        userContext.setId(userId);
        // 记录登录时间以及ip等信息
        userContext.setExtraInfo(SpringWebUtils.getRequest());
        // 登录并缓存用户信息
        StpUtil.login(userId, SaLoginConfig.setExtra(ConstantConfig.USER_CONTENT_EXTRA_KEY, JSON.toJSONString(userContext)));
        CompletableFuture.runAsync(() -> {
            userService.updateById(user);
        });
        return StpUtil.getTokenInfo();
    }
}
