package com.xs.core.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.function.Consumer;

public class ExceptionUtils {
    private static final Logger log = LoggerFactory.getLogger(ExceptionUtils.class);

    private ExceptionUtils() {
    }

    public static void printException(Runnable runnable, Throwable throwable) {
        if (null == throwable && runnable instanceof Future<?> future) {
            try {
                if (future.isDone()) {
                    future.get();
                }
            } catch (CancellationException e) {
                throwable = e;
            } catch (ExecutionException e) {
                throwable = e.getCause();
            } catch (InterruptedException var6) {
                Thread.currentThread().interrupt();
            }
        }

        if (null != throwable) {
            log.error(throwable.getMessage(), throwable);
        }

    }

    public static <T> T exToNull(ExSupplier<T> exSupplier) {
        return (T) exToDefault(exSupplier, null);
    }

    public static <T> T exToNull(ExSupplier<T> supplier, Consumer<Exception> exConsumer) {
        return (T) exToDefault(supplier, null, exConsumer);
    }

    public static String exToBlank(ExSupplier<String> exSupplier) {
        return (String) exToDefault(exSupplier, "");
    }

    public static <T> T exToDefault(ExSupplier<T> exSupplier, T defaultValue) {
        return (T) exToDefault(exSupplier, defaultValue, (Consumer) null);
    }

    public static <T> T exToDefault(ExSupplier<T> exSupplier, T defaultValue, Consumer<Exception> exConsumer) {
        try {
            return exSupplier.get();
        } catch (Exception e) {
            if (null != exConsumer) {
                exConsumer.accept(e);
            }

            return defaultValue;
        }
    }

    @FunctionalInterface
    public interface ExSupplier<T> {
        T get() throws Exception;
    }
}

