package com.xs.core.utils;

import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.extra.spring.SpringUtil;
import jakarta.servlet.ServletContext;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.context.ApplicationContext;
import org.springframework.http.server.PathContainer;
import org.springframework.web.accept.ContentNegotiationManager;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.servlet.HandlerMapping;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.handler.SimpleUrlHandlerMapping;
import org.springframework.web.util.UrlPathHelper;
import org.springframework.web.util.pattern.PathPattern;
import org.springframework.web.util.pattern.PathPatternParser;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;

public class SpringWebUtils {
    private SpringWebUtils() {
    }

    public static HttpServletRequest getRequest() {
        return getServletRequestAttributes().getRequest();
    }

    public static HttpServletResponse getResponse() {
        return getServletRequestAttributes().getResponse();
    }

    public static boolean isMatch(String path, List<String> patterns) {
        return patterns.stream().anyMatch((pattern) -> isMatch(path, pattern));
    }

    public static boolean isMatch(String path, String... patterns) {
        return Arrays.stream(patterns).anyMatch((pattern) -> isMatch(path, pattern));
    }

    public static boolean isMatch(String path, String pattern) {
        PathPattern pathPattern = PathPatternParser.defaultInstance.parse(pattern);
        PathContainer pathContainer = PathContainer.parsePath(path);
        return pathPattern.matches(pathContainer);
    }

    public static void deRegisterResourceHandler(Map<String, String> handlerMap) {
        ApplicationContext applicationContext = SpringUtil.getApplicationContext();
        HandlerMapping resourceHandlerMapping = (HandlerMapping) applicationContext.getBean("resourceHandlerMapping", HandlerMapping.class);
        Map<String, Object> oldHandlerMap = (Map) ReflectUtil.getFieldValue(resourceHandlerMapping, "handlerMap");

        for (Map.Entry<String, String> entry : handlerMap.entrySet()) {
            String pathPattern = CharSequenceUtil.appendIfMissing((CharSequence) entry.getKey(), "/**", new CharSequence[0]);
            oldHandlerMap.remove(pathPattern);
        }

    }

    public static void registerResourceHandler(Map<String, String> handlerMap) {
        ApplicationContext applicationContext = SpringUtil.getApplicationContext();
        HandlerMapping resourceHandlerMapping = (HandlerMapping) applicationContext.getBean("resourceHandlerMapping", HandlerMapping.class);
        Map<String, Object> oldHandlerMap = (Map) ReflectUtil.getFieldValue(resourceHandlerMapping, "handlerMap");
        ServletContext servletContext = (ServletContext) applicationContext.getBean(ServletContext.class);
        ContentNegotiationManager contentNegotiationManager = (ContentNegotiationManager) applicationContext.getBean("mvcContentNegotiationManager", ContentNegotiationManager.class);
        UrlPathHelper urlPathHelper = (UrlPathHelper) applicationContext.getBean("mvcUrlPathHelper", UrlPathHelper.class);
        ResourceHandlerRegistry resourceHandlerRegistry = new ResourceHandlerRegistry(applicationContext, servletContext, contentNegotiationManager, urlPathHelper);

        for (Map.Entry<String, String> entry : handlerMap.entrySet()) {
            String pathPattern = CharSequenceUtil.appendIfMissing((CharSequence) entry.getKey(), "/**", new CharSequence[0]);
            oldHandlerMap.remove(pathPattern);
            String resourceLocations = CharSequenceUtil.appendIfMissing((CharSequence) entry.getValue(), "/", new CharSequence[0]);
            resourceHandlerRegistry.addResourceHandler(new String[]{pathPattern}).addResourceLocations(new String[]{"file:" + resourceLocations});
        }

        Map<String, ?> additionalUrlMap = ((SimpleUrlHandlerMapping) ReflectUtil.invoke(resourceHandlerRegistry, "getHandlerMapping", new Object[0])).getUrlMap();
        ReflectUtil.invoke(resourceHandlerMapping, "registerHandlers", new Object[]{additionalUrlMap});
    }

    private static ServletRequestAttributes getServletRequestAttributes() {
        return (ServletRequestAttributes) Objects.requireNonNull(RequestContextHolder.getRequestAttributes());
    }
}